#-----------------------------------------------------------------------------
# Compatible: SketchUp 7 (PC)
#             (other versions untested)
#-----------------------------------------------------------------------------
#
# CHANGELOG
# 2.4.0 - 10.09.2010
#		 * Added module: TT::Binary
#		 * Added class:  TT::JSON
#		 * Added module: TT::GUI
#		 * Added class:  TT::Window
#		 * Added class:  TT::Inputbox
#		 * Added module: TT::System
#		 * Added module: TT::Locale
#		 * Added module: TT::Cursor
#
# 2.3.0 - 08.09.2010
#		 * Added method: TT::Geom3d.spiral_sphere
#		 * Fixed: TT::Ray.test
#
# 2.2.0 - 06.09.2010
#		 * Added module: TT::Bounds
#
# 2.1.1 - 04.09.2010
#		 * Fixed bug: Ray.test (Workaround for SU8.0 bug)
#
# 2.1.0 - 03.09.2010
#		 * Added module: TT::UVQ
#		 * Added class:  TT::UV_Plane
#		 * Added method: TT::Entities.bounds
#		 * Added class:  TT::Gizmo::Manipulator
#		 * Added module: TT::Selection
#		 * Fixed: TT::Ray.test
#
# 2.0.0 - 01.09.2010
#		 * Initial Release
#
#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------

# Root namespace for Thomas Thomassen (ThomThom, TT)
#
# @since 2.0.0
module TT
  
  ### CONSTANTS ### --------------------------------------------------------
  
  # BoundingBox Constants
  # @since 2.0.0
  
  BB_LEFT_FRONT_BOTTOM    =  0
  BB_RIGHT_FRONT_BOTTOM   =  1
  BB_LEFT_BACK_BOTTOM     =  2
  BB_RIGHT_BACK_BOTTOM    =  3
  BB_LEFT_FRONT_TOP       =  4
  BB_RIGHT_FRONT_TOP      =  5
  BB_LEFT_BACK_TOP        =  6
  BB_RIGHT_BACK_TOP       =  7
  
  BB_CENTER_FRONT_BOTTOM  =  8
  BB_CENTER_BACK_BOTTOM   =  9
  BB_CENTER_FRONT_TOP     = 10
  BB_CENTER_BACK_TOP      = 11
  
  BB_LEFT_CENTER_BOTTOM   = 12
  BB_LEFT_CENTER_TOP      = 13
  BB_RIGHT_CENTER_BOTTOM  = 14
  BB_RIGHT_CENTER_TOP     = 15
  
  BB_LEFT_FRONT_CENTER    = 16
  BB_RIGHT_FRONT_CENTER   = 17
  BB_LEFT_BACK_CENTER     = 18
  BB_RIGHT_BACK_CENTER    = 19
  
  BB_LEFT_CENTER_CENTER   = 20
  BB_RIGHT_CENTER_CENTER  = 21
  BB_CENTER_FRONT_CENTER  = 22
  BB_CENTER_BACK_CENTER   = 23
  BB_CENTER_CENTER_TOP    = 24
  BB_CENTER_CENTER_BOTTOM = 25
  
  BB_CENTER_CENTER_CENTER = 26
  BB_CENTER               = 26
  
  # UI.messagebox Constants
  # @since 2.4.0
  
  MB_ICONHAND         = 0x00000010
  MB_ICONSTOP         = 0x00000010
  MB_ICONERROR        = 0x00000010
  MB_ICONQUESTION     = 0x00000020
  MB_ICONEXCLAMATION  = 0x00000030
  MB_ICONWARNING      = 0x00000030
  MB_ICONASTERISK     = 0x00000040
  MB_ICONINFORMATION  = 0x00000040
  MB_ICON_NONE        = 80
  
  MB_DEFBUTTON1 = 0x00000000
  MB_DEFBUTTON2 = 0x00000100
  MB_DEFBUTTON3 = 0x00000200
  MB_DEFBUTTON4 = 0x00000300
  
  ### LIBRAY ### -----------------------------------------------------------
  
  # TT_Lib related methods.
  #
  # @since 2.0.0
  module Lib
    
    # Library version number.
    # @since 2.0.0
    VERSION = '2.4.0'
    
    # Array index for VERSION.split('.')
		# @since 2.0.0
		V_MAJOR = 0
    
    # Array index for VERSION.split('.')
		# @since 2.0.0
		V_MINOR = 1
    
    # Array index for VERSION.split('.')
		# @since 2.0.0
		V_REV   = 2
    
    
    # Call this method to check if the installed +TT_Lib+ version is the same
    # or newer than +version+. If it's not then a messagebox will appear
    # informing the user that a newer +TT_Lib+ is required.
    #
    # @param [String] version a string with the minimun version required in
    #   the format 'x.x.x'.
    # @param [String] plugin_name a string describing to the user which plugin
    #   require a newer TT_Lib version.
    #
    # @return [Boolean]
    # @since 2.0.0
    def self.compatible?(version, plugin_name = 'A plugin installed')
      lib_v = TT::Lib::VERSION.split('.').map { |s| s.to_i }
      v = version.split('.').map { |s| s.to_i }
      return true if lib_v[V_MAJOR] > v[V_MAJOR]
      return true if lib_v[V_MAJOR] == v[V_MAJOR] && lib_v[V_MINOR] > v[V_MINOR]
      return true if lib_v[V_MAJOR] == v[V_MAJOR] && lib_v[V_MINOR] == v[V_MINOR] && lib_v[V_REV] >= v[V_REV]
      UI.messagebox("#{plugin_name} requires a newer version, #{version}, of TT_Lib.")
      return false
    end
    
    
    # @return [String] The file path where the library is installed.
    # @since 2.0.0
    def self.path
      File.dirname( File.expand_path( __FILE__ ) )
    end
    
    
    # Debug method to reload the library modules.
    #
    # @param [Boolean] return_files Determines if the method should return
    #   the number of files reloaded or an array of the files reloaded.
    #
    # @return [Integer, Array]
    # @since 2.0.0
    def self.reload( return_files=false )
      x = Dir.glob( File.join(self.path, '*.{rb,rbs}') ).each { |file|
        load file
      }
      (return_files) ? x : x.length
    end
  
  end # module TT::Lib
  
  
  ### MENUS ### ------------------------------------------------------------
  
  # If TT's Menu is installed this method will return a custom Menu item
  # instead of the requested root menu.
  #
  # @param [String] name The prefered root menu in Sketchup.
  #
  # @return [Sketchup::Menu]
  # @since 2.0.0
  def self.menu( name )
    ($tt_menu) ? $tt_menu : UI.menu( name )
  end
  
  
  ### NAMESPACE ### --------------------------------------------------------
  
  # Namespace for plugins to be wrapped into.
  # Example:
  #
  #   require 'TT_Lib2/core.rb'
  #   module TT::Plugins::FooBar
  #     ...
  #   end
  #
  # @since 2.0.0
  module Plugins; end
  
  
end # module TT

# Require remaining modules.
Dir.glob( File.join(TT::Lib.path, '*.{rb,rbs}') ).each { |file|
  require file unless File.basename( file ) == 'core.rb'
}