#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'TT_Lib2/window.rb'

# (i) Alpha stage. Very likely to be subject to change!
#
# @example
#   i = TT::GUI::Inputbox.new
#   i.add_control( {
#     :label => 'FooBar',
#     :value => 'Hello'
#   } )
#   i.prompt { |results|
#     p results
#   }
#
# @since 2.4.0
class TT::GUI::Inputbox < TT::GUI::Window
  
  CT_LIST     = 1
  CT_RADIOBOX = 2
  
  # Creates a new Inputbox instance.
  #
  # @todo Escape HTML data (?) Ok on Windows IE.
  # @todo Descriptions (Info, HTML)
  # @todo Slider
  # @todo PickList
  # @todo Events
  #
  # == +options+ Keys
  # * +:title+ (Default: +"Inputbox"+)
  #
  # * +:pref_key+ If present the inputbox will remember it's values and properties
  #   between sessions.
  #
  # * +:save_values+ (Default: +true+) Set to false if you do not want the values
  #   to be remembered between sessions, but want the window size etc to be
  #   remembered. (Requires +:pref_key+ to be +true+)
  #
  # * +:resizable+ (Default: +true+)
  #
  # * +:width+
  #
  # * +:height+
  #
  # * +:accept_label+ (Default: +"Ok"+) The caption of the accept button.
  #
  # * +:cancel_label+ (Default: +"Cancel"+) The caption of the cancel button.
  #
  # * +:modal+ (Default: +false+) Set to true to prevent the user from interacting
  #   with the model while the window is open. It also prevents other modal windows
  #   (From TT_Lib) from opening.
  #
  # * +:true_modal+ (Default: +false+) *Deprecated* Under Windows the window can
  #   be made into a true modal window, but since it can't under OSX this key
  #   is deprecated.
  #
  # @param [Hash] options
  #
	# @since 2.4.0
	def initialize(options={})
    raise ArgumentError unless options.is_a?( Hash )
    
    # Window options (Sent to Window < Webdialog instance)
    wnd_options = {
      :dialog_title => 'Inputbox',
      :scrollable   => false,
      :resizable    => true
    }
    wnd_options[:dialog_title]    = options[:title]     if options.key?(:title)
    wnd_options[:preferences_key] = options[:pref_key]  if options.key?(:pref_key)
    wnd_options[:resizable]       = options[:resizable] if options.key?(:resizable)
    wnd_options[:width]           = options[:width]     if options.key?(:width)
    wnd_options[:height]          = options[:height]    if options.key?(:height)
    # Window UI options (Sent to Javascript)
    html_options = {
      :accept_label => 'Ok',
      :cancel_label => 'Cancel'
    }
    html_options[:save_values]  = options[:save_values] if options.key?(:save_values)
    html_options.merge!(options)
    # Internal flags.
    @save_values  = ( options.key?(:save_values) )  ? options[:save_values] : true
    @true_modal   = ( options.key?(:true_modal) )   ? options[:true_modal]  : false
    @modal        = ( options.key?(:modal) )        ? options[:modal]       : false
    # Array of the controls and their values.
    @controls = []
    @values = nil
    # Flag indicating if the window is closing.
    @closing = false
    # Initate the ModalWrapper
    if !@true_modal && @modal 
      @modal_window = TT::GUI::ModalWrapper.new( self )
    end
    # Control's label is the section key for settings
    if options.key?(:pref_key) && @save_values
      @defaults = TT::Settings.new( options[:pref_key] )
    else
      @defaults = nil
    end
    
    # Initialize the parent Window class
		super(wnd_options)
    self.min_width = 200
    self.min_height = 120
    wpath = File.join( TT::Lib.path, 'webdialog' )
    add_script( File.join(wpath, 'js', 'inputbox.js') )
    add_style( File.join(wpath, 'css', 'inputbox.css') )
		
    # When the webdialog reports it's ready, call function to start building the
    # UI with the given user options.
		self.add_action_callback('Inputbox_ready') { |dialog, params|
			puts '>> Input Ready'
      # Window settings
      o = TT::JSON.new(html_options)
      dialog.execute_script("inputbox.init_html(#{o});")
      # Add controls
      @controls.each { |control|
        c = control.clone
        if c[:value].is_a?( Float ) && !c[:value].is_a?( Length )
          c[:value] = TT::Locale.float_to_string( c[:value] )
        elsif c[:value].is_a?( Length )
          c[:value] = c[:value].to_s
        end
        dialog.execute_script("inputbox.add_control(#{c});")
      }
		}
    
    # When the user accepts the values in the Inputbox, collect the data and
    # convert them back from string into their input types.
    self.add_action_callback('Inputbox_accept') { |dialog, params|
			puts '>> Input Accept'
      @values = @controls.map { |control|
        v = get_value( control[:id] )
        type = ( control[:options] && control[:multiple] ) ? Array : control[:value]
        v = TT::Locale.cast_string(v, type, '||')
        @defaults[ control[:label] ] = v if @save_values
        v
      }
      dialog.close
		}
    
    # User cancels the inputbox, either by using the Cancel button, the Close
    # button or the Right-Click menu.
    self.add_action_callback('Inputbox_cancel') { |dialog, params|
			puts '>> Input Cancel'
      dialog.close
		}
    
    # Flag the window as closing to avoid the Modal_Wrapper from also calling
    # close - something which will lead to multiple triggering of this event.
    self.set_on_close {
      puts '>> Window Closed'
      @closing = true 
      @block.call(@values)
      @modal_window.close if @modal_window
    }
	end
  
  
  # Flag indicating if the webdialog is closing.
  #
  # @since 2.4.0
  def closing?
    @closing
  end
  
  
  # Adds a new input field.
  #
  # == +options+ Keys:
  # * +:label+ (+String+) *Required* Should be unique in order for persistant values to
  #   function.
  #
  # * +:value+ (+String+|+Array+) *Required* Default value.
  #
  # * +:description+ (+String+) Explainatory text accosiated to the control.
  #
  # * +:options+ (+Array+) Array of valued, used for lists and radiobox options.
  #
  # * +:order+ (integer) -1, 0 or 1 - -1 means decending order, 0 no order,
  #   1 accending order.
  #
  # * +:natrual_order+ (+Boolean+) If the list is ordered, set this true for a
  #   natrual ordering.
  #
  # * +:size+ (+Integer+) Set this property for a scrollable list instead of a
  #   drop down list.
  #
  # * +:multiple+ (+Boolean+) Allows mulitple items to be selected.
  #
  # * +:type+ (+Integer+) Possible constants:
  #   * +TT::GUI::Inputbox::CT_LIST+
  #   * +TT::GUI::Inputbox::CT_RADIOBOX+
  #
  # == Normal Textbox
  #  i.add_control( {
  #    :label => 'Hello',
  #    :value => 'World'
  #  } )
  #
  # == Natrually Ordered Multi Select List
  #  i.add_control( {
  #    :label         => 'My List',
  #    :description   => 'Lorem Ipsum Dolor Sit Amet.',
  #    :value         => ['Hello', 'World'],
  #    :options       => ['Foo', 'Hello', 'Bar', 'World', 'FooBar'],
  #    :multiple      => true,
  #    :order         => 1,
  #    :natrual_order => true,
  #    :size          => 5
  #  } )
  #
  # @param [Hash] options
  #
  # @since 2.4.0
  def add_control(options)
    # Ensure the options are JSON object which can be sent to the Webdialog.
    # Hashes will aumatically be converted to JSON objects.
    options = ( options.is_a?( Hash ) ) ? TT::JSON.new( options ) : options.dup
    raise ArgumentError unless options.is_a?( TT::JSON )
    # Process and prepare the options before sending it to the Webdialog.
    options[:id] = "Inputbox_control#{@controls.size}"
    if @defaults
      options[:value] = @defaults[ options[:label], options[:value] ]
    end
    @controls << options
    # If controls are added while the inputbox is open it needs to be notified.
    if self.visible?
      self.execute_script("inputbox.add_control(#{options});")
    end
  end
  
  
  # @param [&block] block the callback that receives the resulting values when
  # the inputbox closes. Only used if the inputbox is not true modal.
  #
  # @return [Array|nil] Array of resulting values if Inputbox is true modal, nil otherwise.
  # @since 2.4.0
  def prompt(&block)
    @block = block
    if @true_modal
      show_window(@true_modal)
      @values
    elsif @modal
      @modal_window.show
      nil
    else
      self.show_window
      nil
    end
  end
  
  
  # @deprecated Debug function
  # @return [String]
  # @since 2.4.0
  def get_html()
    self.execute_script("inputbox.debug();")
    self.get_element_value('RUBY_Inputbox_get_value');
  end
  
  private
  
  
  # Returns the value of the given element identified by its index in +@controls+.
  #
  # @param [String] id
  #
  # @return [String]
  # @since 2.4.0
  def get_value(id)
    self.execute_script("inputbox.get_value('#{id}');")
    self.get_element_value('RUBY_Inputbox_get_value');
  end

end # module TT::GUI::Inputbox