#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

# Wrapper for +Sketchup.read_default+ and +Sketchup.write_default+.
#
# To initialize defaults:
#   @settings = TT::Settings.new('FooBar')
#   @settings[:foo, false]
#   @settings[:bar, true]
#
# @since 2.0.0
class TT::Settings
  
  # Creates a new Settings instance that read and writes values to +section+
  # in SketchUp's preferences.
  #
  # @param [String] section
  #
  # @since 2.0.0
  def initialize(section)
    @section = section
    @cache = {}
  end
  
  
  # Type casts read values based on the default value given where
  # +Sketchup.read_default+ would otherwise not do so.
  #
  # Custom type casts: +Length+, +Symbol+
  #
  # @param [String, Symbol] key
  # @param [mixed] value
  #
  # @return [mixed]
  # @since 2.0.0
  def [](key, default = nil)
    if @cache.key?(key)
      x = @cache[key]
    else
      x = Sketchup.read_default(@section, key.to_s, default)
      x = x.to_l if default.is_a?(Length)
      x = x.intern if x.is_a?(String) && default.is_a?(Symbol)
      @cache[key] = x
      x
    end
  end
  
  
  # Converts +Length+ to +Float+.
  #
  # Converts +Symbol+ to +String+.
  #
  # @param [String, Symbol] key
  # @param [mixed] value
  #
  # @return [mixed]
  # @since 2.0.0
  def []=(key, value)
    @cache[key] = value
    value = value.to_f if value.is_a?(Length)
    value = value.to_s if value.is_a?(Symbol)
    Sketchup.write_default(@section, key.to_s, value)
  end
  
end # class TT::Settings