#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

# ...
#
# @since 2.4.0
class TT::UV_Plane
  
  attr_reader(:origin, :normal)
  attr_accessor(:material, :width, :height)
  
  
  # Creates a new UV Plane.
	#
	# @param [Geom::Point3d] origin
	# @param [Geom::Vector3d] normal
  # @param [Sketchup::Material] material
  #
	# @since 2.4.0
  def initialize(origin, normal, material)
    @material = material
    if @material.texture.nil?
      @width  = 10
      @height = 10
    else
      @width  = @material.texture.width
      @height = @material.texture.height
    end
    
    @origin = origin.clone
    @normal = normal.clone
    @plane = [@origin, @normal]
    update()
  end

  
  # Returns an array representing the UV plane.
  #
	# @return [Array<Geom::Point3d,Geom::Vector3d>]
	# @since 2.4.0
  def to_a
    return [origin, normal]
  end
  

  # Sets the origin of the plane where the UV mapping originates from.
	#
	# @param [Geom::Point3d] point
  #
	# @return [Geom::Point3d]
	# @since 2.4.0
  def origin=(point)
    @origin = point
    update()
    @origin
  end
  
  
  # Sets the normal of the UV plane.
	#
	# @param [Geom::Vector3d] vector
  #
	# @return [Geom::Vector3d]
	# @since 2.4.0
  def normal=(vector)
    @normal = vector
    update()
    @normal
  end
  
  
  # Projects a texture material from the UV plane onto the face.
	#
	# @param [Sketchup::Face] face
	# @param [Boolean] on_front
  #
	# @since 2.4.0
  def project(face, on_front=true)
    # Special case for untextured materials
    if @material.nil? || @material.materialType == 0
      face.material = @material
      return
    end
    
    # The UV mapping co-ords to feed position_material.
    uv_points = []
    
    # Can't rely on the vertex positions, since a face might have only tree
    # and four sets are required to correctly map a distorted texture.
    # And the point can't be co-linear either.
    #
    # Instead, generate four points on the plane of the target face.
    # 3  4
    #
    # 1  2
    pts = []
    pts << face.vertices.first.position               # 1
    pts << pts.first.offset( face.normal.axes.x, 10 ) # 2
    pts << pts.first.offset( face.normal.axes.y, 10 ) # 3
    pts << pts.last.offset(  face.normal.axes.x, 10 ) # 4
    
    # Now project each of the point to the UV plane in order to obtain
    # the UV coordinates relative to the UV origin.
    pts.each { |p1|
      # Get UV coordinates.
      p2 = p1.project_to_plane(@plane)
      uv = get_UV( p2 )
      # Add the points to the pool.
      uv_points << p1
      uv_points << uv
    }
    face.position_material( @material, uv_points, on_front )
  end
  
  
  # Projects a texture material from the UV plane onto the faces in the given
  # collection of entities.
	#
	# @param [Sketchup::Entities,Sketchup::Selection,Array<Sketchup::Entity>] entities
	# @param [Boolean] on_front
  #
	# @since 2.4.0
  def project_to_entities(entities, on_front=true)
    for e in entities
      next unless e.is_a?(Sketchup::Face)
      project( e, on_front )
    end
  end
  
  
  # Calculates the UV coordinates for the given 3d point. The point should be on
  # the UV plane.
  #
  # Ref:
  # http://mathforum.org/library/drmath/view/51727.html
  #
  # Thanks to Chris Thomson for the SU solution.
  # http://forums.sketchucation.com/viewtopic.php?f=180&t=28484&p=247275#p247271
  #
  # @param [Geom::Point3d] point3d
  #
	# @since 2.4.0
  def get_UV( point3d )
    point2d = point3d.transform(@to_local)
    u = point2d.x / @width
    v = point2d.y / @height
    return [u, v, 1.0]
  end
  
  
  # (!)
  def frame_segments
    # return the 3d points representing the material boundary on the plane.
  end
  
  private
  
  # Updates the cached transformation for the UV Plane.
	#
	# @return [Geom::Transformation]
	# @since 2.4.0
  def update
    # Transformation to convert 3d points on the plane to local 2d points.
    o = @origin.offset( @normal.axes.x.reverse, @width / 2.0 )
    o.offset!( @normal.axes.y.reverse, @height / 2.0 )
    x = @normal.axes.x
    y = @normal.axes.y
   # @to_local = Geom::Transformation.axes(@origin, x, y, @normal).inverse
    @to_local = Geom::Transformation.axes(o, x, y, @normal).inverse
  end

end # module TT::UV_Plane